/*
	Copyright 2018-2019 Giovanni Franza - HB9EIK 

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "struct.h"
#include "math.h"
#include <dlfcn.h>
#include <syslog.h>

void initDll( workp works );
void initSetup( workp works );
void logSetup( workp works );
void newButton( workp works, char *buf );
void newCell( workp works, char *buf );
void newBand( workp works, char *buf );
void newEncoder( workp works, char *buf );
void newColor( workp works, char *buf, GLfloat *colors );
void newFont( workp works, char *buf );
void newScreen( workp works, char *buf );
void newSetup( workp works, char *buf );
void LineButtons( workp works );

#define NONE 0
#define BUTTON 1
#define CELL 2
#define BAND 3
#define ENCODER 4
#define COLOR 5
#define FONT 6
#define SCREEN 7
#define SETUP 8

GLfloat colors[(sizeof(color)/sizeof(GLfloat))];

void ReadInit( workp works ) {
	FILE *fd;
	char buf[1000];
	char *pp;
	int j, k;

        fd = fopen( "./init.txt", "r" );
        if ( !fd ) {
                syslog( LOG_ERR, "can't open ./init.txt" );
                return;
        }
        for( ;; ) {
                pp = fgets( buf, sizeof( buf ), fd );
                if( !pp ) {
                        break;
                }
		if( pp[0]=='#' ) {
			continue;
		}
		if( pp[strlen(pp)-1]=='\n' ) {
			pp[strlen(pp)-1]=0;
		}
		if( pp[strlen(pp)-1]=='\r' ) {
			pp[strlen(pp)-1]=0;
		} 
		if( pp[strlen(pp)-1]=='\n' ) {
			pp[strlen(pp)-1]=0;
		}
		if( !strncmp( pp, "X: ", 3 ) ) {
			works->xPos=atoi(buf+3);
			continue;
		} else if( !strncmp( pp, "Y: ", 3 ) ) {
			works->yPos=atoi(buf+3);
			continue;
		} else if( !strncmp( pp, "Width: ", 7 ) ) {
			works->xSize=atoi(buf+7);
			continue;
		} else if( !strncmp( pp, "Height: ", 8 ) ) {
			works->ySize=atoi(buf+8);
			continue;
		} else if( !strncmp( pp, "Passphrase: ", 12 ) ) {
			strcpy( works->passphrase, buf+12 );
			continue;
		} else if( !strncmp( pp, "FFTLevel: ", 10 ) ) {
			strcpy( works->setup->FFTlevel, buf+10 );
			sscanf( works->setup->FFTlevel, "%f", &works->FFTlevel );
			continue;
		} else if( !strncmp( pp, "FFTSpan: ", 9 ) ) {
			strcpy( works->setup->FFTspan, buf+9 );
			sscanf( works->setup->FFTspan, "%f", &works->FFTspan );
			continue;
		} else if( !strncmp( pp, "FFTalpha: ", 10 ) ) {
			works->FFTalpha=atof( pp+10 );
			continue;
		} else if( !strncmp( pp, "FFTtype: ", 9 ) ) {
			works->FFTtype=atoi( pp+9 );
			continue;
		} else if( !strncmp( pp, "FFTdelay: ", 10 ) ) {
			works->FFTdelay=atoi( pp+10 );
			continue;
		} else if( !strncmp( pp, "AuxVolStep: ", 12 ) ) {
			works->auxVolIncr=atoi(buf+12);
			continue;
		} else if( !strncmp( pp, "RitStep: ", 9 ) ) {
			works->freqRitIncr=atoi(buf+9);
			continue;
		} else if( !strncmp( pp, "FreqStep: ", 10 ) ) {
			works->freqIncr=atoi(buf+10);
			continue;
		} else if( !strncmp( pp, "StartLayer: ", 12 ) ) {
			works->setup->startLayer=atoi(buf+12);
			continue;
		} else if( !strncmp( pp, "Antialiased: ", 13 ) ) {
			works->setup->Antialiased=atoi( pp+13 );
		} else if( !strncmp( pp, "Graphic: ", 9 ) ) {
			works->setup->Graphic=atoi( pp+9 );
		} else if( !strncmp( pp, "Server: ", 8 ) ) {
			strcpy( works->setup->server, pp+8 );
			strcpy( works->server, works->setup->server );
		} else if( !strncmp( pp, "TcpPort: ", 9 ) ) {
			sscanf( pp+9, "%hd", &works->port );
		} else if( !strncmp( pp, "IsTouch: True", 13 ) ) {
			works->setup->isTouch=1;
		} else if( !strncmp( pp, "IsTouch: False", 14 ) ) {
			works->setup->isTouch=0;
		} else if( !strncmp( pp, "FifoBaud: ", 10 ) ) {
			works->setup->FifoBaud = atoi( pp+10 );
		} else if( !strncmp( pp, "FifoDevice: ", 12 ) ) {
			strcpy( works->setup->FifoDevice, pp+12 );
		} else if( !strncmp( pp, "KeyerDevice: ", 13 ) ) {
			strcpy( works->setup->KeyerDevice, pp+13 );
		} else if( !strncmp( pp, "AutoSnap: ", 10 ) ) {
			works->setup->AutoSnap = atoi( pp+10 );
		} else if( !strncmp( pp, "BufferingTime: ", 15 ) ) {
			sscanf( pp+15, "%f", &works->setup->bufferingTime );
			works->buffer_start=works->setup->bufferingTime/works->setup->rxChunkTime;
		} else if( !strncmp( pp, "RxChunkTime: ", 13 ) ) {
			sscanf( pp+13, "%f", &works->setup->rxChunkTime );
			works->rx_buffer_frames=works->setup->rxChunkTime*SAMPLES/2.5;
		} else if( !strncmp( pp, "TxChunkTime: ", 13 ) ) {
			sscanf( pp+13, "%f", &works->setup->txChunkTime );
			works->tx_buffer_frames=works->setup->txChunkTime*SAMPLES/2.5;
		} else if( !strncmp( pp, "AudioAuxIn: ", 12 ) ) {
			works->setup->devTxAux = atoi( pp+12 );
		} else if( !strncmp( pp, "AudioIn: ", 9 ) ) {
			works->setup->devTx = atoi( pp+9 );
		} else if( !strncmp( pp, "AudioAuxOut: ", 13 ) ) {
			works->setup->devRxAux = atoi( pp+13 );
		} else if( !strncmp( pp, "AudioOut: ", 10 ) ) {
			works->setup->devRx = atoi( pp+10 );
		} else if( !strncmp( pp, "AudioRxCompress: ", 17 ) ) {
			sscanf( pp+17, "%ld", &works->rxcompress );
		} else if( !strncmp( pp, "AudioTxCompress: ", 17 ) ) {
			sscanf( pp+17, "%ld", &works->txcompress );
		} else if( !strncmp( pp, "AutoReturn: ", 12 ) ) {
			works->setup->autoReturn = atoi( pp+12 );
		} else if( !strncmp( pp, "KeyerSpeed: ", 12 ) ) {
			works->setup->KeyerSpeed = atoi( pp+12 );
		} else if( !strncmp( pp, "AudioFftEnabled: ", 17 ) ) {
			works->setup->audioFftEnabled = atoi( pp+17 );
		} else if( !strncmp( pp, "AuxVol: ", 8 ) ) {
			works->setup->auxVol = atoi( pp+8 );
		} else if( !strncmp( pp, "EnableCW: ", 10 ) ) {
			works->setup->enableCW = pp[10];
		} else if( !strncmp( pp, "VorAuP: ", 8 ) ) {
			works->setup->vorAuP = atoi( pp+8 );
		} else if( !strncmp( pp, "VotAuP: ", 8 ) ) {
			works->setup->votAuP = atoi( pp+8 );
		} else if( !strncmp( pp, "HwDebug: ", 9 ) ) {
			works->setup->hwdebug = atoi( pp+9 );
		} else if( !strncmp( pp, "WhatAuDev: ", 11 ) ) {
			works->setup->whatAuDev = atoi( pp+11 );
		} else if( !strncmp( pp, "KeyerMode: ", 11 ) ) {
			works->setup->KeyerMode = atoi( pp+11 );
		} else if( !strncmp( pp, "EnableTmate: ", 13 ) ) {
			works->setup->enableTmate = pp[13];
			works->notmate = pp[13] == '1' ? 0:1;
		} else if( !strncmp( pp, "Relax: ", 7 ) ) {
			works->setup->relax=atoi( pp+7 );
		} else if( !strncmp( pp, "TmateRxColor: ", 14 ) ) {
			works->setup->rxcolor[0] = atoi( pp+14 );
			works->setup->rxcolor[1] = atoi( pp+19 );
			works->setup->rxcolor[2] = atoi( pp+24 );
		} else if( !strncmp( pp, "TmateTxColor: ", 14 ) ) {
			works->setup->txcolor[0] = atoi( pp+14 );
			works->setup->txcolor[1] = atoi( pp+19 );
			works->setup->txcolor[2] = atoi( pp+24 );
		} else if( !strncmp( pp, "TmateCallbacks: ", 16 ) ) {
			for( k=0; k<12; k++ ) {
				works->setup->t2keys[k] = atoi( pp+16+k*5 );
			}
		} else if( !strncmp( pp, "KbCallbacks: ", 13 ) ) {
			for( k=0; k<12; k++ ) {
				works->setup->kbkeys[k] = atoi( pp+13+k*5 );
			}
		} else if( !strncmp( pp, "Language: ", 10 ) ) {
			strcpy( works->setup->myLangue, pp+10 );
			for( j=0; j<works->nLanguages; j++ ) {
				if( !strcmp( pp+10, works->Languages[j] ) ) {
					works->setup->langue=j;
					break;
				}
			}
		}
	}
	fclose( fd );
}

void WriteInit( workp works ) {
	FILE *fd;

        fd = fopen( "./init.txt", "w" );
        if ( !fd ) {
                syslog( LOG_ERR, "can't open for write ./setup.txt" );
                return;
        }
	fprintf( fd, "X: %d\n", works->xPos );
	fprintf( fd, "Y: %d\n", works->yPos );
	fprintf( fd, "Width: %d\n", works->realxSize );
	fprintf( fd, "Height: %d\n", works->realySize );
	fprintf( fd, "FFTLevel: %4.0f\n", works->FFTlevel );
	fprintf( fd, "FFTSpan: %4.0f\n", works->FFTspan );
	fprintf( fd, "FFTalpha: %f\n", works->FFTalpha );
	fprintf( fd, "FFTtype: %d\n", works->FFTtype );
	fprintf( fd, "FFTdelay: %d\n", works->FFTdelay );
	fprintf( fd, "AuxVolStep: %d\n", works->auxVolIncr );
	fprintf( fd, "RitStep: %d\n", works->freqRitIncr );
	fprintf( fd, "FreqStep: %d\n", works->freqIncr );
	fprintf( fd, "StartLayer: %d\n", works->plotter->baseLayer );
	fprintf( fd, "Antialiased: %d\n", works->setup->Antialiased );
	fprintf( fd, "Graphic: %d\n", works->setup->Graphic );
	fprintf( fd, "IsTouch: %s\n", works->setup->isTouch?"True":"False" );
	fprintf( fd, "Server: %s\n", works->server );
	fprintf( fd, "TcpPort: %d\n", works->port );
	fprintf( fd, "Passphrase: %s\n", works->passphrase );
	fprintf( fd, "AutoSnap: %d\n", works->setup->AutoSnap );
	fprintf( fd, "AudioAuxIn: %d\n", works->setup->devTxAux==works->setup->defTx?-1:works->setup->devTxAux );
	fprintf( fd, "AudioIn: %d\n", works->setup->devTx==works->setup->defTx?-1:works->setup->devTx );
	fprintf( fd, "AudioAuxOut: %d\n", works->setup->devRxAux==works->setup->defRx?-1:works->setup->devRxAux );
	fprintf( fd, "AudioOut: %d\n", works->setup->devRx==works->setup->defRx?-1:works->setup->devRx );
	fprintf( fd, "AudioRxCompress: %ld\n", works->rxcompress );
	fprintf( fd, "AudioTxCompress: %ld\n", works->txcompress );
	fprintf( fd, "RxChunkTime: %f\n", works->setup->rxChunkTime );
	fprintf( fd, "TxChunkTime: %f\n", works->setup->txChunkTime );
	fprintf( fd, "BufferingTime: %f\n", works->setup->bufferingTime );
	fprintf( fd, "AutoReturn: %d\n", works->setup->autoReturn );
	fprintf( fd, "KeyerSpeed: %d\n", works->setup->KeyerSpeed );
	fprintf( fd, "VorAuP: %d\n", works->setup->vorAuP );
	fprintf( fd, "VotAuP: %d\n", works->setup->votAuP );
	fprintf( fd, "WhatAuDev: %d\n", works->setup->whatAuDev );
	fprintf( fd, "HwDebug: %d\n", works->setup->hwdebug );
	fprintf( fd, "KeyerMode: %d\n", works->setup->KeyerMode );
	fprintf( fd, "AuxVol: %d\n", works->setup->auxVol );
	fprintf( fd, "AudioFftEnabled: %d\n", works->setup->audioFftEnabled );
	fprintf( fd, "Language: %s\n", works->setup->myLangue );
	fprintf( fd, "EnableCW: %c\n", works->setup->enableCW );
	fprintf( fd, "FifoBaud: %d\n", works->setup->FifoBaud );
	fprintf( fd, "FifoDevice: %s\n", works->setup->FifoDevice );
	fprintf( fd, "KeyerDevice: %s\n", works->setup->KeyerDevice );
	fprintf( fd, "Relax: %d\n", works->setup->relax );
	fprintf( fd, "EnableTmate: %c\n", works->setup->enableTmate );
	fprintf( fd, "TmateRxColor: %03d, %03d, %03d\n", works->setup->rxcolor[0], works->setup->rxcolor[1], works->setup->rxcolor[2] );
	fprintf( fd, "TmateTxColor: %03d, %03d, %03d\n", works->setup->txcolor[0], works->setup->txcolor[1], works->setup->txcolor[2] );
	fprintf( fd, "TmateCallbacks: %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d\n", works->setup->t2keys[0], works->setup->t2keys[1], works->setup->t2keys[2], works->setup->t2keys[3], works->setup->t2keys[4], works->setup->t2keys[5], works->setup->t2keys[6], works->setup->t2keys[7], works->setup->t2keys[8], works->setup->t2keys[9], works->setup->t2keys[10], works->setup->t2keys[11] );
	fprintf( fd, "KbCallbacks: %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d\n", works->setup->kbkeys[0], works->setup->kbkeys[1], works->setup->kbkeys[2], works->setup->kbkeys[3], works->setup->kbkeys[4], works->setup->kbkeys[5], works->setup->kbkeys[6], works->setup->kbkeys[7], works->setup->kbkeys[8], works->setup->kbkeys[9], works->setup->kbkeys[10], works->setup->kbkeys[11] );
	fclose( fd );
}

void ReadConfig( workp works ) {
	FILE *fd;
	char buf[1000];
	char *pp;
	int element=NONE;
	char *error;
	int j;

	initSetup( works );

	initDll( works );

	works->nbuttons=0;

	for( j=0; j<works->nActions; j++ ) {	
		dlerror();
		works->sintonia = dlsym( works->libActions[j], "sintonia" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "sintonia", error);
		exit(1);
	}

	memset( buf, 0, sizeof( buf ) );
	for( j=0; j<strlen( works->argv0 )-10; j++ ) {
		if( !strncmp( works->argv0+j, "EladRemote", 10 ) ) {
			sprintf( buf, "./setup%s.txt", works->argv0+j+10 );
			break;
		}
	}
	if( j>=strlen( works->argv0 )-10 ) { 
		strcpy( buf, "./setup.txt" );
	}
	fd = fopen( buf, "r" );
        if ( !fd ) {
                syslog( LOG_ERR, "can't open %s", buf );
                exit(1);
        }
        for( ;; ) {
                pp = fgets( buf, sizeof( buf ), fd );
                if( !pp ) {
                        break;
                }
		if( pp[0]=='#' ) {
			continue;
		}
		if( !strncmp( pp, "BUTTONS {", 9 ) ) {
			element=BUTTON;
			continue;
		}
		if( !strncmp( pp, "CELLS {", 7 ) ) {
			element=CELL;
			continue;
		}
		if( !strncmp( pp, "BANDS {", 7 ) ) {
			element=BAND;
			continue;
		}
		if( !strncmp( pp, "ENCODERS {", 10 ) ) {
			element=ENCODER;
			continue;
		}
		if( !strncmp( pp, "COLORS {", 8 ) ) {
			element=COLOR;
			continue;
		}
		if( !strncmp( pp, "FONTS {", 7 ) ) {
			element=FONT;
			continue;
		}
		if( !strncmp( pp, "SCREEN {", 8 ) ) {
			element=SCREEN;
			continue;
		}
		if( !strncmp( pp, "SETUP {", 7 ) ) {
			element=SETUP;
			continue;
		}
		if( !strncmp( pp, "};", 2 ) ) {
			element=NONE;
			continue;
		}
		switch( element ) {
			case BUTTON:
				newButton( works, buf );
				break;
			case CELL:
				newCell( works, buf );
				break;
			case BAND:
				newBand( works, buf );
				break;
			case ENCODER:
				newEncoder( works, buf );
				break;
			case COLOR:
				newColor( works, buf, colors );
				break;
			case FONT:
				newFont( works, buf );
				break;
			case SCREEN:
				newScreen( works, buf );
				break;
			case SETUP:
				newSetup( works, buf );
				break;
		}
        }
	fclose( fd );
	memcpy( works->colors, colors, sizeof( colors ) );
	ReadInit( works );
	if( works->setup->Graphic ) {
		works->setup->Antialiased=0;
	}
	logSetup( works );
	LineButtons( works );
	works->plotter->layer=1;
	works->setup->sidetone=0;
	works->keyFd=-1;
}
void newButton( workp works, char *buf ) {		
	int j, jj;
	char *qq;
	int k, l, m;
	char * error;

	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]==';' ) {
			buf[k]=0;
			switch( m ) {
				case  0: // int button number
					j=atoi(buf+l)-1;
					if( j==0 ) {
						works->buttons=(buttonp)calloc( 1, sizeof(button));
						if( !works->buttons ) {
							syslog( LOG_ERR, "buttons calloc (%d) failed", j+1 );
							exit( 1 );
						}
					} else {
						if( j>= works->nbuttons ) {
							works->buttons=(buttonp)realloc( works->buttons, sizeof(button)*(j+1) );
							if( !works->buttons ) {
								syslog( LOG_ERR, "buttons calloc (%d) failed", j+1 );
								exit( 1 );
							}
						} 
						memset( works->buttons+j, 0, sizeof(button) );
					}
					(works->buttons+j)->tleft=0;
					(works->buttons+j)->ttop=0;
					(works->buttons+j)->twidth=0;
					(works->buttons+j)->theight=0;
					(works->buttons+j)->left=0;
					(works->buttons+j)->top=0;
					(works->buttons+j)->width=0;
					(works->buttons+j)->height=0;
					(works->buttons+j)->pressed=0;
					(works->buttons+j)->modified=-1;
					(works->buttons+j)->over=0;
					(works->buttons+j)->isEncoder=0;
					break;
				case  1: // int toggle;
					(works->buttons+j)->toggle=atoi(buf+l);
					break;
				case  2:	// int activated;
					if( buf[l]=='\'' ) {
						(works->buttons+j)->activated=buf[l+1];
					} else {
						(works->buttons+j)->activated=atoi(buf+l);
					}
					break;
				case  3:	// char *str1;
					if( strncmp( buf+l, "NULL", 4 ) ) {
						strcpy( (works->buttons+j)->str1, buf+l );
					} else {
						(works->buttons+j)->str1[0]=0;
					}
					break;
				case  4:	// char *str2;
					if( strncmp( buf+l, "NULL", 4 ) ) {
						strcpy( (works->buttons+j)->str2, buf+l );
					} else {
						(works->buttons+j)->str2[0]=0;
					}
					break;
				case  5:	// void (*write)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->write = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->write = NULL;
					}
					break;
				case  6:	// void (*read)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->read = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->read = NULL;
					}
					break;
				case  7:	// void (*down)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->down = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->down = NULL;
					}
					break;
				case  8:	// void (*up)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->up = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->up = NULL;
					}
					break;
				case  9:	// char group;
					(works->buttons+j)->group=atoi(buf+l);
					break;
				case 10:	// char *name;
					buf[l+strlen(buf+l)-1]=0;
					l++;
					if( strncmp( buf+l, "NULL", 4 ) ) {
						qq = calloc( strlen( buf+l )+1, 1 );
						if( !qq ) {
							syslog( LOG_ERR, "buttons name calloc (%d) failed", j+1 );
							exit( 1 );
						}
						strcpy( qq, buf+l );
					} else {
						qq=NULL;
					}
					(works->buttons+j)->name=qq;
					break;
				case 11:	// int  band;
					(works->buttons+j)->band=atoi(buf+l);
					break;
				case 12:	// int color;
					(works->buttons+j)->color=atoi(buf+l);
					break;
				case 13:	// char **decode;
					break;
				case 14:	// unsigned char layer;
					(works->buttons+j)->layer=atoi(buf+l);
					break;
				case 15:	// int transparent;
					(works->buttons+j)->transparent=atoi(buf+l);
					break;
				case 16:	// void (*dxdn)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->dxdn = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->dxdn = NULL;
					}
					break;
				case 17:	// Tmate interested
					(works->buttons+j)->tmate=atoi(buf+l);
					if( (works->buttons+j)->tmate ) {
						if( works->tmateNbuttons==0 ) {
							works->tmateButtons=( int *)calloc( 1, sizeof(int));
							if( !works->tmateButtons ) {
								syslog( LOG_ERR, "tmatebuttons calloc (%d) failed", j+1 );
								exit( 1 );
							}
						} else {
							works->tmateButtons=( int *)realloc( works->tmateButtons, sizeof(int)*(works->tmateNbuttons+1) );
							if( !works->tmateButtons ) {
								syslog( LOG_ERR, "tmatebuttons calloc (%d) failed", j+1 );
								exit( 1 );
							}
						}
						works->tmateButtons[works->tmateNbuttons]=j+1;
						works->tmateNbuttons++;
					}
					break;
				case 18:	// void (*centerdn)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->centerdn = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->centerdn = NULL;
					}
			}
			m++;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
	}
	j++;
	if( works->nbuttons<j ) {
		works->nbuttons=j;
	}
}
void newCell( workp works, char *buf ) {		
	static int j=0;
	int k, l, m;

	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]=='{' ) {
			if( j==0 ) {
				works->cells=(cellp)calloc( 1, sizeof(button));
			} else {
				works->cells=(cellp)realloc( works->cells, sizeof(cell)*(j+1) );
				memset( works->cells+j, 0, sizeof(cell) );
			}
			if( !works->cells ) {
				syslog( LOG_ERR, "cells calloc (%d) failed", j+1 );
				exit( 1 );
			}
			m=0;
			buf[k]=0;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
		if( buf[k]==',' || buf[k]=='}' ) {
			buf[k]=0;
			switch( m ) {
				case  0: // float row;
					(works->cells+j)->row=atof(buf+l);
					break;
				case  1:	// float col;
					(works->cells+j)->col=atof(buf+l);
				case  2:	// float rowspan;
					(works->cells+j)->rowspan=atof(buf+l);
					break;
				case  3:	// float colspan;
					(works->cells+j)->colspan=atof(buf+l);
					break;
				case  4:	// int button;
					if( buf[l]=='\'' ) {
						(works->cells+j)->button=buf[l+1];
					} else {
						(works->cells+j)->button=atoi(buf+l);
					}
					j++;
			}
			m++;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
	}
}
void newBand( workp works, char *buf ) {		
	static int j=0;
	int k, l, m;

	if( j==0 ) {
		works->bands=(bandp)calloc( 1, sizeof(band));
		works->imposts=(impostp)calloc( 1, sizeof(impost));
	} else {
		works->bands=(bandp)realloc( works->bands, sizeof(band)*(j+1) );
		memset( works->bands+j, 0, sizeof(band) );
		works->imposts=(impostp)realloc( works->imposts, sizeof(impost)*(j+1) );
		memset( works->imposts+j, 0, sizeof(impost) );
	}
	works->nbands=j+1;
	if( !works->bands ) {
		syslog( LOG_ERR, "bands calloc (%d) failed", j+1 );
		exit( 1 );
	}
	if( !works->imposts ) {
		syslog( LOG_ERR, "imposts calloc (%d) failed", j+1 );
		exit( 1 );
	}
	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]==',' ) {
			buf[k]=0;

			switch( m ) {
				case  0: // int start;
					(works->bands+j)->start=atoi(buf+l);
					break;
				case  1:	// int stop;
					(works->bands+j)->stop=atoi(buf+l);
					break;
				case  2:	// int call;
					(works->bands+j)->call=atoi(buf+l);
					(works->imposts+j)->freq=atoi(buf+l);
					break;
				case  3:	// int mode;
					(works->bands+j)->mode=atoi(buf+l);
					(works->imposts+j)->mode=atoi(buf+l);
					break;
				case  4:	// int bw;
					(works->bands+j)->bw=atoi(buf+l);
					(works->imposts+j)->bw=atoi(buf+l);
					break;
				case  5:	// int lpf;
					(works->bands+j)->lpf=atoi(buf+l);
					(works->imposts+j)->lpf=atoi(buf+l);
					break;
				case  6:	// int antOut;
					(works->bands+j)->antOut=atoi(buf+l);
					(works->imposts+j)->antOut=atoi(buf+l);
					break;
				case  7:	// int button;
					(works->bands+j)->button=atoi(buf+l);
					break;
			}
			m++;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
	}
	(works->imposts+j)->att=-1;
	(works->imposts+j)->nr=-1;
	(works->imposts+j)->nb=-1;
	(works->imposts+j)->an=-1;
	(works->imposts+j)->rit=-1;
	(works->imposts+j)->an=-1;
	(works->imposts+j)->agcAuto=-1;
	(works->imposts+j)->agcAlvl=-1;
	(works->imposts+j)->agcMlvl=-1;
	(works->imposts+j)->squelch=-1;
	(works->imposts+j)->antOut=-1;
	(works->imposts+j)->sat=0;
	j++;
}

void newEncoder( workp works, char *buf ) {		
	int j, jj;
	char *qq;
	int k, l, m;
	char * error;

	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]==';' ) {
			buf[k]=0;
			switch( m ) {
				case  0:	// encoder number
					j=atoi(buf+l)-1;
					if( j==0 ) {
						works->buttons=(buttonp)calloc( 1, sizeof(button));
						if( !works->buttons ) {
							syslog( LOG_ERR, "encoder calloc (%d) failed", j+1 );
							exit( 1 );
						}
					} else {
						if( j >= works->nbuttons ) {
							works->buttons=(buttonp)realloc( works->buttons, sizeof(button)*(j+1) );
							if( !works->buttons ) {
								syslog( LOG_ERR, "encoder calloc (%d) failed", j+1 );
								exit( 1 );
							}
						}
						memset( works->buttons+j, 0, sizeof(button) );
					}
					(works->buttons+j)->tleft=0;
					(works->buttons+j)->ttop=0;
					(works->buttons+j)->twidth=0;
					(works->buttons+j)->theight=0;
					(works->buttons+j)->left=0;
					(works->buttons+j)->top=0;
					(works->buttons+j)->width=0;
					(works->buttons+j)->height=0;
					(works->buttons+j)->pressed=0;
					(works->buttons+j)->modified=-1;
					(works->buttons+j)->over=0;
					(works->buttons+j)->isEncoder=1;
					(works->buttons+j)->str1[0]=0;
					(works->buttons+j)->str2[0]=0;
					break;
				case  1:	// void (*write)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->write = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->write = NULL;
					}
					break;
				case  2:	// void (*read)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							dlerror();
							(works->buttons+j)->read = dlsym( works->libActions[jj], buf+l );
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->read = NULL;
					}
					break;
				case  3:	// void (*wheelDn)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							(works->buttons+j)->wheelDn = dlsym( works->libActions[jj], buf+l );
							dlerror();
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->wheelDn = NULL;
					}
					break;
				case  4:	// void (*wheelUp)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							(works->buttons+j)->wheelUp = dlsym( works->libActions[jj], buf+l );
							dlerror();
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->wheelUp = NULL;
					}
					break;
				case  5:	// void (*down)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							(works->buttons+j)->down = dlsym( works->libActions[jj], buf+l );
							dlerror();
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->down = NULL;
					}
					break;
				case  6:	// void (*up)( void *, int );
					if( strncmp( buf+l, "NULL", 4 ) ) {
						for( jj=0; jj<works->nActions; jj++ ) {	
							(works->buttons+j)->up = dlsym( works->libActions[jj], buf+l );
							dlerror();
							if ((error = dlerror()) == NULL)  {
								break;
							}
						}
						if ( error != NULL )  {
							syslog( LOG_ERR, "error retrieving function '%s': %s", buf+l, error);
							exit(1);
						}
					} else {
						(works->buttons+j)->up = NULL;
					}
					break;
				case  7:	// char **decode;
					break;
				case  8:	// char *name;
					buf[l+strlen(buf+l)-1]=0;
					l++;
					if( strncmp( buf+l, "NULL", 4 ) ) {
						qq = calloc( strlen( buf+l )+1, 1 );
						if( !qq ) {
							syslog( LOG_ERR, "buttons name calloc (%d) failed", j+1 );
							exit( 1 );
						}
						strcpy( qq, buf+l );
					} else {
						qq=NULL;
					}
					(works->buttons+j)->name=qq;
					break;
				case  9:	// int  transparent;
					(works->buttons+j)->transparent=atoi(buf+l);
					break;
				case 10:	// int  layer;
					(works->buttons+j)->layer=atoi(buf+l);
					break;
				case 11:	// int color;
					(works->buttons+j)->color=atoi(buf+l);
					break;
				case 12:	// int level;
					(works->buttons+j)->level=atoi(buf+l);
					break;
				case 13:	// int size;
					(works->buttons+j)->size=atoi(buf+l);
					break;
				case 14:	// reserved
					break;
				case 15:	// Tmate interested
					(works->buttons+j)->tmate=atoi(buf+l);
					if( (works->buttons+j)->tmate ) {
						if( works->tmateNencoders==0 ) {
							works->tmateEncoders=(int *)calloc( 1, sizeof(int));
							if( !works->tmateEncoders ) {
								syslog( LOG_ERR, "tmateencoders calloc (%d) failed", j+1 );
								exit( 1 );
							}
						} else {
							works->tmateEncoders=(int *)realloc( works->tmateEncoders, sizeof(int)*(works->tmateNencoders+1) );
							if( !works->tmateEncoders ) {
								syslog( LOG_ERR, "tmateencoders calloc (%d) failed", j+1 );
								exit( 1 );
							}
						}
						works->tmateEncoders[works->tmateNencoders]=j+1;
						works->tmateNencoders++;
					}
					break;
			}
			m++;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
	}
	j++;
	if( works->nbuttons<j ) {
		works->nbuttons=j;
	}
}

void newFont( workp works, char *buf ) {		
	static int j=0;
	int k, l, m;
	int faceSize, fontSize, fontDepth;
	float outSet1, outSet2;

	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]==',' ) {
			buf[k]=0;
			switch( m ) {
				case  0:	// fontname
					buf[l+strlen(buf+l)-1]=0;
					l++;
					works->font[j] = ftglCreatePolygonFont( buf+l );
					if( !works->font[j] ) {
						syslog( LOG_ERR, "error retrieving texture font '%s'", buf+l );
						exit(1);
					}
					works->font[j+1] = ftglCreateTextureFont( buf+l );
					if( !works->font[j+1] ) {
						syslog( LOG_ERR, "error retrieving polygon font '%s'", buf+l );
						exit(1);
					}
					break;
				case  1:	// faceSize
					faceSize=atoi( buf+l );
					break;
				case  2:	// fontSize
					fontSize=atoi( buf+l );
					ftglSetFontFaceSize( works->font[j], faceSize, fontSize );
					ftglSetFontFaceSize( works->font[j+1], faceSize, fontSize );
					break;
				case  3:	// fontDepth
					fontDepth=atoi( buf+l );
					ftglSetFontDepth( works->font[j+1], fontDepth );
					break;
				case  4:	// outSet1
					outSet1=atof( buf+l );
					break;
				case  5:	// outSet2
					outSet2=atof( buf+l );
					ftglSetFontOutset( works->font[j], outSet1, outSet2 );
					ftglSetFontCharMap( works->font[j], ft_encoding_unicode );
					ftglSetFontOutset( works->font[j+1], outSet1, outSet2 );
					ftglSetFontCharMap( works->font[j+1], ft_encoding_unicode );
					break;
			}
			m++;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
	}
	j+=2;
}

void newScreen( workp works, char *buf ) {		
	int k, l, m;

	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]==',' ) {
			buf[k]=0;
			switch( m ) {
				case  0:	// Ssize
					works->xSize=atoi( buf+l );
				case  1:	// ySize
					works->ySize=atoi( buf+l );
					break;
			}
			m++;
			l=k+1;
			while( buf[l]==' ' ) {
				l++;
			}
		}
	}
}

void newSetup( workp works, char *buffer ) {
	int j, k;

	if( buffer[strlen(buffer)-1]=='\n' ) {
		buffer[strlen(buffer)-1]=0;
	}
	if( buffer[strlen(buffer)-1]=='\r' ) {
		buffer[strlen(buffer)-1]=0;
	} 
	if( !strncmp( buffer, "passphrase: ", 12 ) ) {
		strcpy( works->passphrase, buffer+12 );
	} else if( !strncmp( buffer, "EnableRead: ", 12 ) ) {
		works->setup->enableRead = buffer[12];
	} else if( !strncmp( buffer, "EnableWrite: ", 13 ) ) {
		works->setup->enableWrite = buffer[13];
	} else if( !strncmp( buffer, "EnableTmate: ", 13 ) ) {
		works->setup->enableTmate = buffer[13];
		works->notmate = buffer[13] == '1' ? 0:1;
	} else if( !strncmp( buffer, "EnableAudioFFT: ", 16 ) ) {
		works->setup->enableAudioFFT = buffer[16];
	} else if( !strncmp( buffer, "EnableCW: ", 10 ) ) {
		works->setup->enableCW = buffer[10];
	} else if( !strncmp( buffer, "EnableFIFO: ", 12 ) ) {
		works->setup->enableFIFO = buffer[12];
	} else if( !strncmp( buffer, "AutoReturn: ", 12 ) ) {
		works->setup->autoReturn = atoi( buffer+12 );
	} else if( !strncmp( buffer, "KeyerSpeed: ", 12 ) ) {
		works->setup->KeyerSpeed = atoi( buffer+12 );
	} else if( !strncmp( buffer, "VotAuP: ", 8 ) ) {
		works->setup->votAuP = atoi( buffer+8 );
	} else if( !strncmp( buffer, "VorAuP: ", 8 ) ) {
		works->setup->vorAuP = atoi( buffer+8 );
	} else if( !strncmp( buffer, "WhatAuDev: ", 11 ) ) {
		works->setup->whatAuDev = atoi( buffer+11 );
	} else if( !strncmp( buffer, "HwDebug: ", 9 ) ) {
		works->setup->hwdebug = atoi( buffer+9 );
	} else if( !strncmp( buffer, "KeyerMode: ", 11 ) ) {
		works->setup->KeyerMode = atoi( buffer+11 );
	} else if( !strncmp( buffer, "AuxVol: ", 8 ) ) {
		works->setup->auxVol = atoi( buffer+8 );
	} else if( !strncmp( buffer, "AudioFftEnabled: ", 17 ) ) {
		works->setup->audioFftEnabled = atoi( buffer+17 );
	} else if( !strncmp( buffer, "FifoBaud: ", 10 ) ) {
		works->setup->FifoBaud = atoi( buffer+10 );
	} else if( !strncmp( buffer, "FifoDevice: ", 12 ) ) {
		strcpy( works->setup->FifoDevice, buffer+12 );
	} else if( !strncmp( buffer, "KeyerDevice: ", 13 ) ) {
		strcpy( works->setup->KeyerDevice, buffer+13 );
	} else if( !strncmp( buffer, "EnableTime: ", 12 ) ) {
		works->setup->enableTime = buffer[12];
	} else if( !strncmp( buffer, "WebSocket: ", 11 ) ) {
		works->setup->webSocket = buffer[11];
	} else if( !strncmp( buffer, "FFTspan: ", 9 ) ) {
		strcpy( works->setup->FFTspan, buffer+9 );
		sscanf( works->setup->FFTspan, "%f", &works->FFTspan );
	} else if( !strncmp( buffer, "FFTalpha: ", 10 ) ) {
		works->FFTalpha=atof( buffer+10 );
	} else if( !strncmp( buffer, "FFTlevel: ", 10 ) ) {
		strcpy( works->setup->FFTlevel, buffer+10 );
		sscanf( works->setup->FFTlevel, "%f", &works->FFTlevel );
	} else if( !strncmp( buffer, "FFTtype: ", 9 ) ) {
		works->FFTtype=atoi( buffer+9 );
	} else if( !strncmp( buffer, "FFTdelay: ", 10 ) ) {
		works->FFTdelay=atoi( buffer+10 );
	} else if( !strncmp( buffer, "AudioAuxIn: ", 12 ) ) {
		works->setup->devTxAux = atoi( buffer+12 );
	} else if( !strncmp( buffer, "AudioIn: ", 9 ) ) {
		works->setup->devTx = atoi( buffer+9 );
	} else if( !strncmp( buffer, "InChannels: ", 12 ) ) {
		works->setup->in_channels = atoi( buffer+12 );
	} else if( !strncmp( buffer, "OutChannels: ", 13 ) ) {
		works->setup->out_channels = atoi( buffer+13 );
	} else if( !strncmp( buffer, "AudioRxCompress: ", 17 ) ) {
		sscanf( buffer+17, "%ld", &works->rxcompress );
	} else if( !strncmp( buffer, "AudioTxCompress: ", 17 ) ) {
		sscanf( buffer+17, "%ld", &works->txcompress );
	} else if( !strncmp( buffer, "BufferingTime: ", 15 ) ) {
		sscanf( buffer+15, "%f", &works->setup->bufferingTime );
		works->buffer_start=works->setup->bufferingTime/works->setup->rxChunkTime;
	} else if( !strncmp( buffer, "RxChunkTime: ", 13 ) ) {
		sscanf( buffer+13, "%f", &works->setup->rxChunkTime );
		works->rx_buffer_frames=works->setup->rxChunkTime*SAMPLES/2.5;
	} else if( !strncmp( buffer, "TxChunkTime: ", 13 ) ) {
		sscanf( buffer+13, "%f", &works->setup->txChunkTime );
		works->tx_buffer_frames=works->setup->txChunkTime*SAMPLES/2.5;
	} else if( !strncmp( buffer, "AudioAuxOut: ", 13 ) ) {
		works->setup->devRxAux = atoi( buffer+13 );
	} else if( !strncmp( buffer, "AudioOut: ", 10 ) ) {
		works->setup->devRx = atoi( buffer+10 );
	} else if( !strncmp( buffer, "Server: ", 8 ) ) {
		strcpy( works->setup->server, buffer+8 );
		strcpy( works->server, works->setup->server );
	} else if( !strncmp( buffer, "TcpPort: ", 9 ) ) {
		sscanf( buffer+9, "%hd", &works->port );
	} else if( !strncmp( buffer, "BareFFT: ", 9 ) ) {
		strcpy( works->setup->BareFFT, buffer+9 );
	} else if( !strncmp( buffer, "FlippedFFT: ", 12 ) ) {
		strcpy( works->setup->FlippedFFT, buffer+12 );
	} else if( !strncmp( buffer, "AutoSnap: ", 10 ) ) {
		works->setup->AutoSnap = atoi( buffer+10 );
	} else if( !strncmp( buffer, "TimerDelay: ", 12 ) ) {
		works->setup->timeDelay=atoi( buffer+12 );
	} else if( !strncmp( buffer, "OutSound: ", 10 ) ) {
		works->outSound=atoi( buffer+10 );
	} else if( !strncmp( buffer, "SolidBackground: ", 17 ) ) {
		works->setup->SolidBackground=atoi( buffer+17 );
	} else if( !strncmp( buffer, "Native CW: ", 11 ) ) {
		works->setup->nativeCW=atoi( buffer+11 );
	} else if( !strncmp( buffer, "Antialiased: ", 13 ) ) {
		works->setup->Antialiased=atoi( buffer+13 );
	} else if( !strncmp( buffer, "Graphic: ", 9 ) ) {
		works->setup->Graphic=atoi( buffer+9 );
	} else if( !strncmp( buffer, "StartLayer: ", 12 ) ) {
		works->setup->startLayer=atoi( buffer+12 );
	} else if( !strncmp( buffer, "BareLayer: ", 11 ) ) {
		works->setup->bareLayer=atoi( buffer+11 );
	} else if( !strncmp( buffer, "AudioBuffers: ", 14 ) ) {
		works->setup->AudioBuffers=atoi( buffer+14 );
	} else if( !strncmp( buffer, "End: ", 5 ) ) {
		works->setup->End=atoi( buffer+5 );
	} else if( !strncmp( buffer, "WsockTimeout: ", 14 ) ) {
		works->setup->wsockTimeout=atoi( buffer+14 );
	} else if( !strncmp( buffer, "IsTouch: True", 13 ) ) {
		works->setup->isTouch=1;
	} else if( !strncmp( buffer, "IsTouch: False", 14 ) ) {
		works->setup->isTouch=0;
	} else if( !strncmp( buffer, "Relax: ", 7 ) ) {
		works->setup->relax=atoi( buffer+7 );
	} else if( !strncmp( buffer, "TmateRxColor: ", 14 ) ) {
		works->setup->rxcolor[0] = atoi( buffer+14 );
		works->setup->rxcolor[1] = atoi( buffer+19 );
		works->setup->rxcolor[2] = atoi( buffer+24 );
	} else if( !strncmp( buffer, "TmateTxColor: ", 14 ) ) {
		works->setup->txcolor[0] = atoi( buffer+14 );
		works->setup->txcolor[1] = atoi( buffer+19 );
		works->setup->txcolor[2] = atoi( buffer+24 );
	} else if( !strncmp( buffer, "ErrorCleanTime: ", 16 ) ) {
		works->setup->errorCleanTime = atoi( buffer+16 );
	} else if( !strncmp( buffer, "MaxCleaning: ", 13 ) ) {
		works->setup->maxCleaning = atoi( buffer+13 );
	} else if( !strncmp( buffer, "TmateCallbacks: ", 16 ) ) {
		for( k=0; k<12; k++ ) {
			works->setup->t2keys[k] = atoi( buffer+16+k*5 );
		}
	} else if( !strncmp( buffer, "KbCallbacks: ", 13 ) ) {
		for( k=0; k<12; k++ ) {
			works->setup->kbkeys[k] = atoi( buffer+13+k*5 );
		}
	} else if( !strncmp( buffer, "Language: ", 10 ) ) {
		strcpy( works->setup->myLangue, buffer+10 );
		for( j=0; j<works->nLanguages; j++ ) {
			if( !strcmp( buffer+10, works->Languages[j] ) ) {
				works->setup->langue=j;
				break;
			}
		}
	}				
}

void logSetup( workp works ) {
	syslog( LOG_INFO, "EnableRead: -%c-", works->setup->enableRead );
	syslog( LOG_INFO, "EnableWrite: -%c-", works->setup->enableWrite );
	syslog( LOG_INFO, "EnableCW: -%c-", works->setup->enableCW );
	syslog( LOG_INFO, "EnableFIFO: -%c-", works->setup->enableFIFO );
	syslog( LOG_INFO, "EnableAudioFFT: -%c-", works->setup->enableAudioFFT );
	syslog( LOG_INFO, "EnableTmate: -%c-", works->setup->enableTmate );
	syslog( LOG_INFO, "EnableTime: -%c-", works->setup->enableTime );
	syslog( LOG_INFO, "WebSocket: -%c-", works->setup->webSocket );
	syslog( LOG_INFO, "FFTlevel: -%s-", works->setup->FFTlevel );
	syslog( LOG_INFO, "FFTlevel: -%f-", works->FFTlevel );
	syslog( LOG_INFO, "FFTspan: -%s-", works->setup->FFTspan );
	syslog( LOG_INFO, "FFTspan: -%f-", works->FFTspan );
	syslog( LOG_INFO, "FFTalpha: -%f-", works->FFTalpha );
	syslog( LOG_INFO, "FFTtype: -%d-", works->FFTtype );
	syslog( LOG_INFO, "FFTdelay: -%d-", works->FFTdelay );
	syslog( LOG_INFO, "AudioAuxIn: -%d-", works->setup->devTxAux );
	syslog( LOG_INFO, "AudioIn: -%d-", works->setup->devTx );
	syslog( LOG_INFO, "InChannels: -%d-", works->setup->in_channels );
	syslog( LOG_INFO, "OutChannels: -%d-", works->setup->out_channels );
	syslog( LOG_INFO, "AudioRxCompress: -%ld-", works->rxcompress );
	syslog( LOG_INFO, "AudioTxCompress: -%ld-", works->txcompress );
	syslog( LOG_INFO, "RxChunkTime: -%f-", works->setup->rxChunkTime );
	syslog( LOG_INFO, "TxChunkTime: -%f-", works->setup->txChunkTime );
	syslog( LOG_INFO, "BufferingTime: -%f-", works->setup->bufferingTime );
	syslog( LOG_INFO, "AudioAuxOut: -%d-", works->setup->devRxAux );
	syslog( LOG_INFO, "AudioOut: -%d-", works->setup->devRx );
	syslog( LOG_INFO, "Server: -%s-", works->setup->server );
	syslog( LOG_INFO, "TcpPort: -%d-", works->port );
	syslog( LOG_INFO, "BareFFT: -%s-", works->setup->BareFFT );
	syslog( LOG_INFO, "FlippedFFT: -%s-", works->setup->FlippedFFT );
	syslog( LOG_INFO, "AutoSnap: -%d-", works->setup->AutoSnap );
	syslog( LOG_INFO, "TimeDelay: -%d-", works->setup->timeDelay );
	syslog( LOG_INFO, "SolidBackground: -%d-", works->setup->SolidBackground );
	syslog( LOG_INFO, "OutSound: -%d-", works->outSound );
	syslog( LOG_INFO, "Native CW: -%d-", works->setup->nativeCW );
	syslog( LOG_INFO, "Antialiased: -%d-", works->setup->Antialiased );
	syslog( LOG_INFO, "Graphic: -%d-", works->setup->Graphic );
	syslog( LOG_INFO, "StartLayer: -%d-", works->setup->startLayer );
	syslog( LOG_INFO, "BareLayer: -%d-", works->setup->bareLayer );
	syslog( LOG_INFO, "AudioBuffers: -%d-", works->setup->AudioBuffers );
	syslog( LOG_INFO, "WsockTimeout: -%d-", works->setup->wsockTimeout );
	syslog( LOG_INFO, "IsTouch: -%s-", works->setup->isTouch?"True":"False" );
	syslog( LOG_INFO, "Relax: -%d-", works->setup->relax );
	syslog( LOG_INFO, "AutoReturn: -%d-", works->setup->autoReturn );
	syslog( LOG_INFO, "KeyerSpeed: -%d-", works->setup->KeyerSpeed );
	syslog( LOG_INFO, "KeyerMode: -%d-", works->setup->KeyerMode );
	syslog( LOG_INFO, "VorAuP: -%d-", works->setup->vorAuP );
	syslog( LOG_INFO, "VotAuP: -%d-", works->setup->votAuP );
	syslog( LOG_INFO, "WhatAuDev: -%d-", works->setup->whatAuDev );
	syslog( LOG_INFO, "HwDebug: -%d-", works->setup->hwdebug );
	syslog( LOG_INFO, "AuxVol: -%d-", works->setup->auxVol );
	syslog( LOG_INFO, "AudioFftEnabled: %d\n", works->setup->audioFftEnabled );
	syslog( LOG_INFO, "FifoDevice: -%s-", works->setup->FifoDevice );
	syslog( LOG_INFO, "FifoBaud: -%d-", works->setup->FifoBaud );
	syslog( LOG_INFO, "KeyerDevice: -%s-", works->setup->KeyerDevice );
	syslog( LOG_INFO, "ErrorCleanTime: -%d-", works->setup->errorCleanTime );
	syslog( LOG_INFO, "MaxCleaning: -%d-", works->setup->maxCleaning );
	syslog( LOG_INFO, "End: -%d-", works->setup->End );
	syslog( LOG_INFO, "Language: -%d %s-", works->setup->langue, works->setup->myLangue );
	syslog( LOG_INFO, "TmateRxColor: -%03d, %03d, %03d-", works->setup->rxcolor[0], works->setup->rxcolor[1], works->setup->rxcolor[2] );
	syslog( LOG_INFO, "TmateTxColor: -%03d, %03d, %03d-", works->setup->txcolor[0], works->setup->txcolor[1], works->setup->txcolor[2] );
	syslog( LOG_INFO, "TmateCallbacks: -%03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d-", works->setup->t2keys[0], works->setup->t2keys[1], works->setup->t2keys[2], works->setup->t2keys[3], works->setup->t2keys[4], works->setup->t2keys[5], works->setup->t2keys[6], works->setup->t2keys[7], works->setup->t2keys[8], works->setup->t2keys[9], works->setup->t2keys[10], works->setup->t2keys[11] );
	syslog( LOG_INFO, "KbCallbacks: -%03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d, %03d-", works->setup->kbkeys[0], works->setup->kbkeys[1], works->setup->kbkeys[2], works->setup->kbkeys[3], works->setup->kbkeys[4], works->setup->kbkeys[5], works->setup->kbkeys[6], works->setup->kbkeys[7], works->setup->kbkeys[8], works->setup->kbkeys[9], works->setup->kbkeys[10], works->setup->kbkeys[11] );
}

void initSetup( workp works ) {
	int j;
	works->rx_buffer_frames=SAMPLES*2; // 5ms
	works->tx_buffer_frames=SAMPLES*2; // 5ms
	//works->format = SND_PCM_FORMAT_S16_LE;
	works->rate = 48000;
	works->port = 6666;
	strcpy( works->passphrase, "porompompero" );
	memset( works->setup, 0, sizeof( setup ) );
	memset( works->setup, '1', sizeof( setup )-1 );
	strcpy( works->setup->FFTlevel, "-120" );
	works->FFTlevel=-120;
	strcpy( works->setup->FFTspan, "80" );
	works->FFTspan=80;
	works->FFTalpha=2;
	works->FFTtype=2;
	works->FFTdelay=4;
	works->setup->devTx = -1;
	works->setup->in_channels =  1;
	works->setup->out_channels =  2;
	works->rxcompress = 0;
	works->txcompress = 0;
	works->setup->rxChunkTime=5;
	works->setup->txChunkTime=5;
	works->setup->bufferingTime=20;
	works->buffer_start=4;
	works->setup->devRx = -1;
	strcpy( works->setup->server, "127.0.0.1" );
	strcpy( works->setup->BareFFT, "0" );
	strcpy( works->setup->FlippedFFT, "0" );
	works->setup->AutoSnap=1000;
	works->setup->timeDelay=50000;
	works->setup->SolidBackground=0;
	works->setup->Antialiased=0;
	works->setup->Graphic=1;
	works->setup->isTouch=0;
	works->outSound=0;
	works->delay=10;
	works->ramp=200;
	works->txCwLock=0;
	works->setup->relax=2500;
	works->setup->langue=0;
	works->setup->autoReturn=0;
	strcpy( works->setup->myLangue, "Default" );
	for( j=0; j<works->nLanguages; j++ ) {
		if( !strcmp( works->setup->myLangue, works->Languages[j] ) ) {
			works->setup->langue=j;
			break;
		}
	}
	works->outfifo=-1;
	works->setup->cwKeyPort=0;
	works->setup->fiKeyPort=0;
}

void newColor( workp works, char *buf, GLfloat *colors ) {
	static int j=0;
	int k, l, m;

	for( k=0, l=0, m=0; buf[k] ;k++ ) {
		if( buf[k]==',' ) {
			buf[k]=0;
			colors[j*4+m]=atof(buf+l);
			m++;
			l=k+1;
		}
	}
	j++;
}

void initDll( workp works ) {
	int j;
	char *error;

	for( j=0; j<works->nDrawings; j++ ) {
		works->etichetta = dlsym( works->libDrawings[j], "etichetta" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "sintonia", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawBack = dlsym( works->libDrawings[j], "DrawBack" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawBack", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->FormatFreq = dlsym( works->libDrawings[j], "FormatFreq" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "FormatFreq", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawTune = dlsym( works->libDrawings[j], "DrawTune" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawTune", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawAudioFFT = dlsym( works->libDrawings[j], "DrawAudioFFT" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawAudioFFT", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawFFT = dlsym( works->libDrawings[j], "DrawFFT" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawFFT", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->ConvertMeasColor = dlsym( works->libDrawings[j], "ConvertMeasColor" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "ConvertMeasColor", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawWfall = dlsym( works->libDrawings[j], "DrawWfall" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawWfall", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawScale = dlsym( works->libDrawings[j], "DrawScale" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawScale", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawPwr = dlsym( works->libDrawings[j], "DrawPwr" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawPwr", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawRSSI = dlsym( works->libDrawings[j], "DrawRSSI" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawRSSI", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawArrows = dlsym( works->libDrawings[j], "DrawArrows" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawEncoder = dlsym( works->libDrawings[j], "DrawEncoder" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawEncoder", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawButton = dlsym( works->libDrawings[j], "DrawButton" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawButton", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawTmate = dlsym( works->libDrawings[j], "DrawTmate" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawTmate", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawData = dlsym( works->libDrawings[j], "DrawData" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawData", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawQTC = dlsym( works->libDrawings[j], "DrawQTC" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawQTC", error);
		exit(1);
	}
	for( j=0; j<works->nDrawings; j++ ) {
		works->DrawOTH = dlsym( works->libDrawings[j], "DrawOTH" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "DrawOTH", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->modeChange = dlsym( works->libActions[j], "modeChange" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "modeChange", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->AddCipher = dlsym( works->libActions[j], "AddCipher" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "AddCipher", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->setType = dlsym( works->libActions[j], "setType" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "setType", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->setDelay = dlsym( works->libActions[j], "setDelay" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "setDelay", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->setAB = dlsym( works->libActions[j], "setAB" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "setAB", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->cwTx = dlsym( works->libActions[j], "cwTx" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "cwTx", error);
		exit(1);
	}
	for( j=0; j<works->nActions; j++ ) {	
		works->forget = dlsym( works->libActions[j], "forget" );
		if ((error = dlerror()) == NULL)  {
			break;
		}
	}
	if ( error != NULL )  {
		syslog( LOG_ERR, "error retrieving function '%s': %s", "forget", error);
		exit(1);
	}
}

void LineButtons( workp works ) {
	int j;
	int ncols=0, nrows=0;
	int height, width;
	int hborder=0, vborder=0;
	int btnumber;
	cellp cells=works->cells;
	buttonp buttons=works->buttons;
	for( j=0; cells[j].row>=0; j++ ) {
		if( nrows < 100 *( cells[j].row + cells[j].rowspan ) ) {
			nrows = 100 * (cells[j].row + cells[j].rowspan );
		}
		if( ncols < 100 * ( cells[j].col + cells[j].colspan ) ) {
			ncols = 100 * ( cells[j].col + cells[j].colspan) ;
		}
	}
	height=600*100/nrows;
	width=800*100/ncols;
	//vborder=(600-height*nrows/100)/2;
	//hborder=(800-width*ncols/100)/2;
	//vborder-=3; // for sfondo2
	//hborder-=1; // for sfondo2
	for( j=0; cells[j].row>=0; j++ ) {
		btnumber=cells[j].button;
		if( btnumber==1001 ) { // Scale Expert
			works->plotter->ScaleaXpos=hborder+width*cells[j].col+1;
			works->plotter->ScaleaXsize=width*cells[j].colspan-1;
			works->plotter->ScaleaYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->ScaleaYsize=height*cells[j].rowspan-3;
			works->plotter->ScaleXpos=hborder+width*cells[j].col+1;
			works->plotter->ScaleXsize=width*cells[j].colspan-1;
			works->plotter->ScaleYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->ScaleYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1002 ) { // Scale Bare
			works->plotter->ScalebXpos=hborder+width*cells[j].col+1;
			works->plotter->ScalebXsize=width*cells[j].colspan-1;
			works->plotter->ScalebYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->ScalebYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1003 ) { // Freq position Expert
			works->plotter->FreqaXpos=hborder+width*cells[j].col+1;
			works->plotter->FreqaXsize=width*cells[j].colspan-1;
			works->plotter->FreqaYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->FreqaYsize=height*cells[j].rowspan-3;
			works->plotter->FreqXpos=hborder+width*cells[j].col+1;
			works->plotter->FreqXsize=width*cells[j].colspan-1;
			works->plotter->FreqYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->FreqYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1004 ) { // Freq position Bare
			works->plotter->FreqbXpos=hborder+width*cells[j].col+1;
			works->plotter->FreqbXsize=width*cells[j].colspan-1;
			works->plotter->FreqbYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->FreqbYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1005 ) { // Sintony Expert
			works->plotter->SyntaXpos=hborder+width*cells[j].col+1;
			works->plotter->SyntaXsize=width*cells[j].colspan-1;
			works->plotter->SyntaYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->SyntaYsize=height*cells[j].rowspan-1;
			works->plotter->SyntXpos=hborder+width*cells[j].col+1;
			works->plotter->SyntXsize=width*cells[j].colspan-1;
			works->plotter->SyntYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->SyntYsize=height*cells[j].rowspan-1;
		} else if( btnumber==1006 ) { // Sintony Bare
			works->plotter->SyntbXpos=hborder+width*cells[j].col+1;
			works->plotter->SyntbXsize=width*cells[j].colspan-1;
			works->plotter->SyntbYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->SyntbYsize=height*cells[j].rowspan-1;
		} else if( btnumber==1007 ) { // FFT Expert
			works->plotter->FFTaXpos=hborder+width*cells[j].col+1;
			works->plotter->FFTaXsize=width*cells[j].colspan-2;
			works->plotter->FFTaYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->FFTaYsize=height*cells[j].rowspan;
			works->plotter->FFTXpos=hborder+width*cells[j].col+1;
			works->plotter->FFTXsize=width*cells[j].colspan-2;
			works->plotter->FFTYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->FFTYsize=height*cells[j].rowspan;
		} else if( btnumber==1008 ) { // FFT Bare
			works->plotter->FFTbXpos=hborder+width*cells[j].col+1;
			works->plotter->FFTbXsize=width*cells[j].colspan-2;
			works->plotter->FFTbYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->FFTbYsize=height*cells[j].rowspan;
		} else if( btnumber==1009 ) { // Waterfall Expert
			works->plotter->WfallaXpos=hborder+width*cells[j].col+1;
			works->plotter->WfallaXsize=width*cells[j].colspan-1;
			works->plotter->WfallaYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->WfallaYsize=height*cells[j].rowspan-2;
			works->plotter->WfallXpos=hborder+width*cells[j].col+1;
			works->plotter->WfallXsize=width*cells[j].colspan-1;
			works->plotter->WfallYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->WfallYsize=height*cells[j].rowspan-2;
		} else if( btnumber==1010 ) { // Waterfall Bare
			works->plotter->WfallbXpos=hborder+width*cells[j].col+1;
			works->plotter->WfallbXsize=width*cells[j].colspan-1;
			works->plotter->WfallbYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->WfallbYsize=height*cells[j].rowspan-2;
		} else if( btnumber==1011 ) { // Smeter Expert
			works->plotter->RSSIaXpos=hborder+width*cells[j].col+1;
			works->plotter->RSSIaXsize=width*cells[j].colspan-1;
			works->plotter->RSSIaYpos=600-(vborder+height*cells[j].row+7);
			works->plotter->RSSIaYsize=height*cells[j].rowspan-3;
			works->plotter->RSSIXpos=hborder+width*cells[j].col+1;
			works->plotter->RSSIXsize=width*cells[j].colspan-1;
			works->plotter->RSSIYpos=600-(vborder+height*cells[j].row+7);
			works->plotter->RSSIYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1012 ) { // Smeter Bare
			works->plotter->RSSIbXpos=hborder+width*cells[j].col+1;
			works->plotter->RSSIbXsize=width*cells[j].colspan-1;
			works->plotter->RSSIbYpos=600-(vborder+height*cells[j].row+7);
			works->plotter->RSSIbYsize=height*cells[j].rowspan-3;
			works->plotter->FFTinset=46;
		} else if( btnumber==1013 ) { // MSG
			works->plotter->MSGXpos=hborder+width*cells[j].col+1;
			works->plotter->MSGXsize=width*cells[j].colspan-1;
			works->plotter->MSGYpos=600-(vborder+height*cells[j].row+7);
			works->plotter->MSGYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1014 ) { // Ampli QTC
			works->plotter->OTHXpos=hborder+width*cells[j].col+1;
			works->plotter->OTHXsize=width*cells[j].colspan-1;
			works->plotter->OTHYpos=600-(vborder+height*cells[j].row+7);
			works->plotter->OTHYsize=height*cells[j].rowspan-3;
		} else if( btnumber==1015 ) { // MOD
			works->plotter->MODXpos=hborder+width*cells[j].col+1;
			works->plotter->MODXsize=width*cells[j].colspan-1;
			works->plotter->MODYpos=600-(vborder+height*cells[j].row+7);
			works->plotter->MODYsize=height*cells[j].rowspan-6;
		} else if( btnumber==1016 ) { // Audio FFT Expert 
			works->plotter->AudioFFTaXpos=hborder+width*cells[j].col+1;
			works->plotter->AudioFFTXpos=hborder+width*cells[j].col+1;
			works->plotter->AudioFFTaXsize=width*cells[j].colspan-2;
			works->plotter->AudioFFTXsize=width*cells[j].colspan-2;
			works->plotter->AudioFFTaYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->AudioFFTYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->AudioFFTaYsize=height*cells[j].rowspan;
			works->plotter->AudioFFTYsize=height*cells[j].rowspan;
		} else if( btnumber==1017 ) { // Audio FFT Bare
			works->plotter->AudioFFTbXpos=hborder+width*cells[j].col+1;
			works->plotter->AudioFFTbXsize=width*cells[j].colspan-2;
			works->plotter->AudioFFTbYpos=600-(vborder+height*cells[j].row+4);
			works->plotter->AudioFFTbYsize=height*cells[j].rowspan;
		} else if( btnumber>0 ) {
			btnumber--;
			if( btnumber<works->nbuttons ) {
				buttons[btnumber].top=600-(vborder+height*cells[j].row);
				buttons[btnumber].left=hborder+width*cells[j].col;
				buttons[btnumber].height=height*cells[j].rowspan;
				buttons[btnumber].width=width*cells[j].colspan;
				buttons[btnumber].ttop=600-height*cells[j].row;
				buttons[btnumber].tleft=width*cells[j].col+1;
				buttons[btnumber].theight=height*cells[j].rowspan-1;
				buttons[btnumber].twidth=width*cells[j].colspan;
			}
		}
	}
	for( j=0; j<works->nbuttons; j++ ) {
		if( buttons[j].band >=0 ) {
			works->bands[buttons[j].band].button=j;
		}
	}
}
