/*
	Copyright 2018-2019 Giovanni Franza - HB9EIK 

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <syslog.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <termio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#define XtNumber(x) sizeof(x)/sizeof(x[0])

#include "struct.h"


void *fifo( void *a ) {

	workp works =(workp)a;
	struct termios TtyAttr;
	int rc;
	char buf[100];
	int c;
	int j;
	int ByteBits = CS8;
	struct timeval timeout;
	fd_set set;

	works->outfifo=-1;

	syslog( LOG_INFO, "Fifo thread started" );
	prctl( PR_SET_NAME, (unsigned long)"Fifo", 0, 0, 0 );

	works->fifoFdDef = open( "/dev/ptmx", O_RDWR );
	syslog( LOG_INFO, "works->fifoFdDef=%d opening ptmx\n", works->fifoFdDef );
	if( works->fifoFdDef <=0 ) {
		return( NULL );
	}
	rc = grantpt( works->fifoFdDef );
	if( rc < 0 ) {
		syslog( LOG_ERR, "Error granting ptmx %d", rc );
		return( NULL );
	}
	syslog( LOG_INFO, "grantpt returns %d", rc );

	memset( &TtyAttr, 0, sizeof( struct termios ) );
	tcgetattr( works->fifoFdDef, &TtyAttr ); 
	tcflush( ((workp)works)->fifoFdDef, TCIFLUSH );
	TtyAttr.c_lflag &= ~(ECHO);
        TtyAttr.c_iflag = IGNPAR;
	TtyAttr.c_cflag = ByteBits | CREAD | CLOCAL;
        TtyAttr.c_cc[VMIN] = 1;
	tcsetattr( works->fifoFdDef, TCSANOW, &TtyAttr );
	cfsetispeed( &TtyAttr, ((workp)works)->setup->FifoBaud );
	cfsetospeed( &TtyAttr, ((workp)works)->setup->FifoBaud );
	if( tcsetattr( ((workp)works)->fifoFdDef, TCSANOW, &TtyAttr ) < 0 ) {
		syslog( LOG_ERR, "Unable to set comm port %s\n", ((workp)works)->setup->FifoDevice );
	}

	unlockpt( works->fifoFdDef );
	works->slavename = ptsname( works->fifoFdDef );
	syslog( LOG_INFO, "slavename: %s", works->slavename );
	if( works->setup->hwdebug ) {
		fprintf( stderr, "slavename: %s", works->slavename );
		fflush( stderr );
	}
	rc = chmod( works->slavename, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH );
	if( rc ) {
		syslog( LOG_ERR, "error %d setting permissions on %s", errno, works->slavename );
		if( works->setup->hwdebug ) {
			fprintf( stderr, "error %d sessing permissions on: %s\n", errno, works->slavename );
			fflush( stderr );
		}
	}

	// continuous loop
	works->fifoFd=works->fifoFdDef;
	for( j=0; works->unterminated; ) {		
		for( ;; ) {
			FD_ZERO( &set ); 
			FD_SET( works->fifoFd, &set );
			timeout.tv_sec = 0;
			timeout.tv_usec = 10000;
			rc = select( works->fifoFd+1, &set, NULL, NULL, &timeout );
			if( rc < 0 ) {
				break;
			} 
			if( rc == 0 ) {
				usleep( 10000 );
				continue;
			} 
			rc=read( works->fifoFd, &c, 1 );
			if( rc == 1 ) {
				break;
			}
		}
		if( rc <= 0 ) {
			continue;
		}
		buf[j++]=c;
		if( j>=sizeof( buf ) ) {
			memset( buf, 0, sizeof( buf ) );
			j=0;
			syslog( LOG_ERR, "CAT fifo buffer size exceeded" );
			continue;
		}
		if( c==';' ) {
			buf[j]=0;
			sendWebSockCatFrame( works, buf, strlen(buf) );
			j=0;
			if( works->setup->hwdebug ) {
				fprintf( stderr, "CAT command received %s\n", buf );
				fflush( stderr );
			}
			
		}
	}
	
	close( works->fifoFd );
	syslog( LOG_INFO, "Fifo operations end\n" );
	pthread_exit( a );
	return( a );
}

