/*
	Copyright 2018-2019 Giovanni Franza - HB9EIK 

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <dlfcn.h>
#include <math.h>
#include "portaudio.h"

void MyError(const char *file, int line, const char *function, int err, const char *fmt,...) {
	return;
}

int main(void) {
	int	j, numDevices, defaultDisplayed;
	const PaDeviceInfo *deviceInfo;
	PaStreamParameters inputParameters, outputParameters;
	PaError err; 
	void *dll;
	void (*func)( void * );

	dll = dlopen( "libasound.so", RTLD_LAZY);
	if( dll ) {
		dlerror();
		func = dlsym( dll, "snd_lib_error_set_handler" );
		if( dlerror() == NULL ) {
			func( MyError );
		}
	}
	
	err = Pa_Initialize();
	if( err != paNoError ) {
		printf( "ERROR: Pa_Initialize returned 0x%x\n", err );
		Pa_Terminate();
		return 0;
	}
	
	numDevices = Pa_GetDeviceCount();
	if( numDevices < 0 ) {
		printf( "ERROR: Pa_GetDeviceCount returned 0x%x\n", numDevices );
		err = numDevices;
		Pa_Terminate();
		return 0;
	}
	
	printf( "Input devices\n---------------\n" );
	for( j=0; j<numDevices; j++ ) {
		deviceInfo = Pa_GetDeviceInfo( j );
		if( deviceInfo->maxInputChannels==0 ) {
			continue;
		}
		printf( "%2d ", j );
		defaultDisplayed = 0;
		if( j == Pa_GetDefaultInputDevice() ) {
			printf( "[Default Input]" );
		} else if( j == Pa_GetHostApiInfo( deviceInfo->hostApi )->defaultInputDevice ) {
			const PaHostApiInfo *hostInfo = Pa_GetHostApiInfo( deviceInfo->hostApi );
			printf( "[Default %s Input]", hostInfo->name );
		}
		if( j == Pa_GetDefaultOutputDevice() ) {
			printf( "[Default Output]" );
		} else if( j == Pa_GetHostApiInfo( deviceInfo->hostApi )->defaultOutputDevice ) {
			const PaHostApiInfo *hostInfo = Pa_GetHostApiInfo( deviceInfo->hostApi );
			printf( "[Default %s Output]", hostInfo->name );
		}
		printf( "%s %d %d\n", deviceInfo->name, deviceInfo->maxInputChannels, deviceInfo->maxOutputChannels );
	}

	printf( "Output devices\n---------------\n" );
	for( j=0; j<numDevices; j++ ) {
		deviceInfo = Pa_GetDeviceInfo( j );
		if( deviceInfo->maxOutputChannels==0 ) {
			continue;
		}
		printf( "%2d ", j );
		defaultDisplayed = 0;
		if( j == Pa_GetDefaultInputDevice() ) {
			printf( "[Default Input]" );
		} else if( j == Pa_GetHostApiInfo( deviceInfo->hostApi )->defaultInputDevice ) {
			const PaHostApiInfo *hostInfo = Pa_GetHostApiInfo( deviceInfo->hostApi );
			printf( "[Default %s Input]", hostInfo->name );
		}
		if( j == Pa_GetDefaultOutputDevice() ) {
			printf( "[Default Output]" );
		} else if( j == Pa_GetHostApiInfo( deviceInfo->hostApi )->defaultOutputDevice ) {
			const PaHostApiInfo *hostInfo = Pa_GetHostApiInfo( deviceInfo->hostApi );
			printf( "[Default %s Output]", hostInfo->name );
		}
		printf( "%s %d %d\n", deviceInfo->name, deviceInfo->maxInputChannels, deviceInfo->maxOutputChannels );
	}
	Pa_Terminate();
	return 0;
}
