/*
	Copyright 2018-2019 Giovanni Franza - HB9EIK 

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <png.h>
#include <errno.h>

#include "struct.h"

void readPng( workp works, char *picture, GLubyte *meter, int *Height, int *Width ) {
	int j, k, l;
	int width, height;
	png_byte bit_depth;
	png_byte color_type;
	png_structp png_ptr;
	png_infop info_ptr;
	png_bytep * row_pointers;
	char header[8];  
	FILE *fp;
	char nome[200];

	memset( nome, 0, sizeof(nome ) );
	sprintf( nome, "%s.png", picture );

	fp = fopen( nome, "rb" );
	if( !fp ) {
		syslog( LOG_ERR, "%s image could not be opened", nome );
		return;
	}

	fread( header, 1, 8, fp );
	if( png_sig_cmp( (png_bytep)header, 0, 8 ) ) {
		syslog( LOG_ERR, "%s image is not a PNG file", nome );
		return;
	}

	png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL );
	if( !png_ptr ) {
		syslog( LOG_ERR, "%s PNG_create_read_struct failed", nome );
		return;
	}

	info_ptr = png_create_info_struct( png_ptr );
	if( !info_ptr ) {
		syslog( LOG_ERR, "%s PNG_create_info_struct failed", nome );
		return;
	}

	if( setjmp( png_jmpbuf( png_ptr ) ) ) {
		syslog( LOG_ERR, "%s error during init_io", nome );
		return;
	}
	png_init_io( png_ptr, fp );
	png_set_sig_bytes( png_ptr, 8 );

	png_read_info( png_ptr, info_ptr );

	width = png_get_image_width( png_ptr, info_ptr );
	height = png_get_image_height( png_ptr, info_ptr );
	bit_depth = png_get_bit_depth( png_ptr, info_ptr );
        color_type = png_get_color_type(png_ptr, info_ptr);

	if( width != 400 && width != 200 && width != 100 ) {
		syslog( LOG_ERR, "%s image has %d cols not %d", nome, width, 400 );
		return;
	}
	*Width=width;
	if( height != 240 && height != 120 && height != 60 ) {
		syslog( LOG_ERR, "%s image has %d rows not %d", nome, height, 240 );
		return;
	}
	*Height=height;
	if( bit_depth != 8 || color_type != PNG_COLOR_TYPE_RGB_ALPHA) {
		syslog( LOG_ERR, "%s image has %d pixels not %d", nome, bit_depth, 4 );
		return;
	}

	png_read_update_info( png_ptr, info_ptr );

	if( setjmp( png_jmpbuf( png_ptr ) ) ) {
		syslog( LOG_ERR, "%s error during reading image", nome );
		return;
	}

	row_pointers = (png_bytep*) malloc( sizeof( png_bytep ) * height );
	if( !row_pointers ) {
		syslog( LOG_ERR, "%s unable to alloc row_pointers for image", nome );
		return;
	}
	for (j=0; j<height; j++) {
		row_pointers[j] = (png_byte*) malloc( png_get_rowbytes( png_ptr, info_ptr ) );
		if( !row_pointers[j] ) {
			syslog( LOG_ERR, "%s unable to alloc row_pointers[%d] for image", nome, j );
			return;
		}
	}

	png_read_image( png_ptr, row_pointers );
	for( j=0; j<height; j++ ) {
		for( k=0, l=0; k<width*4; k+=4, l+=3 ) {
			*(meter+j*width*3+l+0) = row_pointers[j][k+0];
			*(meter+j*width*3+l+1) = row_pointers[j][k+1];
			*(meter+j*width*3+l+2) = row_pointers[j][k+2];
		}
	}
	for (j=0; j<height; j++) {
		free( row_pointers[j] );
	}
	free( row_pointers );

	fclose(fp);
}

