/*
	Copyright 2018-2019 Giovanni Franza - HB9EIK 

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <time.h>
#include <math.h>
#include <syslog.h>
#include "struct.h"

void storeFFT( workp works, int DataLength, unsigned char *rbuf) {
	static float Ydata[4*1024]={0};
	static float Ymem[4*1024]={0};
	static int oldDataLength=0;
	int j;
	if( oldDataLength != DataLength ) {
		for(j=0;j<DataLength; j++ ) {
			Ymem[j]=((float)rbuf[j])-192.0;
		}
		oldDataLength=DataLength;
	}
	for( j=0; j<XtNumber(Ydata) && j< DataLength; j++ ) {
		if( works->FFTtype > 0 ) {
			Ydata[j]=(works->FFTalpha*Ymem[j]+((float)rbuf[j])-192.0)/(works->FFTalpha+1);
		} else {
			Ydata[j]=(float)rbuf[j]-192.0;
		}
		Ymem[j]=Ydata[j];
	}
	if( DataLength==1024 ) {
		works->plotter->Ylength = 1024-85-85;
		works->plotter->Ystart = 85;
	} else {
		works->plotter->Ylength = DataLength;
		works->plotter->Ystart = 0;
	}
	memcpy( works->plotter->Ydata, Ydata, sizeof( Ydata ) );
}
void decodeCATcmd( workp works, int DataLength, unsigned char *rbuf) {
	if( !strncmp( (const char *)rbuf, "FP ", 2 ) ) {
		works->cat->fwdPwr=atof( ((const char *)rbuf)+3 );
	} else if( !strncmp( (const char *)rbuf, "RP ", 2 ) ) {
		works->cat->refPwr=atof( ((const char *)rbuf)+3 );
	} else if( !strncmp( (const char *)rbuf, "AN", 2 ) ) {
		works->AntN=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "MG", 2 ) ) {
		works->MGain=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "CG0", 3 ) ) {
		works->CGain=atoi( ((const char *)rbuf)+3 );
	} else if( !strncmp( (const char *)rbuf, "FD", 2 ) ) {
		works->FMdev=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "CI", 2 ) ) {
		works->KePad=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "MT", 2 ) ) {
		works->CWMut=rbuf[2]-'0';
		works->SSBMu=rbuf[3]-'0';
	} else if( !strncmp( (const char *)rbuf, "OS", 2 ) ) {
		works->Offset=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "TE", 2 ) ) {
		works->TXEna=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "TI", 2 ) ) {
		works->TXIn=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "TT", 2 ) ) {
		works->TXOut=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "TU", 2 ) ) {
		works->TuTO=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "TQ", 2 ) ) {
		works->TXPwr=atoi( ((const char *)rbuf)+2 );
		if( works->TXPwr==0 ) {
			works->TXPwr=5100;
		}
	} else if( !strncmp( (const char *)rbuf, "TL", 2 ) ) {
		works->TuPwr=atoi( ((const char *)rbuf)+2 );
	} else if( !strncmp( (const char *)rbuf, "TB", 2 ) ) {
		works->TXBW=atoi( ((const char *)rbuf)+3 );
	}
	works->cat->ros=(1+(sqrt(works->cat->refPwr/works->cat->fwdPwr)))/(1-(sqrt(works->cat->refPwr/works->cat->fwdPwr)));
}
void storeLinear( workp works, int DataLength, unsigned char *rbuf) {
	static int isFirst=1;
	char qq[10];
	if( isFirst ) {
		memset( works->linParms, 0, sizeof( linearParms ) ); 
		isFirst = 0;
	}
	memcpy( works->linParms->banda, rbuf, 4 );
	memset( qq, 0, sizeof(qq) );
	qq[0]=rbuf[4];
	works->linParms->stato=atoi( qq );
	qq[0]=rbuf[5];
	works->linParms->txAtt=atoi( qq );
	if( works->linParms->txAtt != 4 ) {
		works->linParms->oldTxAtt=atoi( qq );
	}
	qq[0]=rbuf[6];
	works->linParms->antenna=atoi( qq );
	qq[0]=rbuf[7];
	works->linParms->standBy=atoi( qq );
	if( works->linParms->standBy ) {
		works->linParms->txAtt=4;
	} 
	if( works->linParms->standBy == 0 ) {
		works->linParms->pa=1;
	} else {
		works->linParms->pa=0;
	}
	qq[0]=rbuf[8];
	if( rbuf[8]=='B' ) {
		works->linParms->atu=0;
	} else if( rbuf[8]=='T' ) {
		works->linParms->atu=1;
	} else if( rbuf[8]=='N' ) {
		works->linParms->atu=2;
	} 
	qq[0]=rbuf[9];
	works->linParms->tune=atoi( qq );
	if( works->linParms->tune ) {
		works->linParms->atu=3;
	}
	memcpy(qq, rbuf+10, 6);
	works->linParms->swr=atof( qq );
	memcpy(qq, rbuf+16, 6);
	works->linParms->pwrOut=atof( qq );
	memcpy(qq, rbuf+22, 6);
	works->linParms->pwrIn=atof( qq );
	memset( qq, 0, sizeof(qq) );
	memcpy(qq, rbuf+28, 4);
	works->linParms->temperatura=atof( qq );
	memcpy(qq, rbuf+32, 4);
	works->linParms->tensione=atof( qq );
	memcpy(qq, rbuf+36, 4);
	works->linParms->corrente=atof( qq );
	memset( qq, 0, sizeof(qq) );
	memcpy(qq, rbuf+40, 2);
	works->linParms->bank=atoi( qq );
	memset( qq, 0, sizeof(qq) );
	if( rbuf[42] == 48 ) {
		works->linParms->errore=0;
	} else {
		works->linParms->errore=rbuf[42];
	}
	memset( works->linParms->versions, 0 , sizeof( works->linParms->versions ) );
	strncpy( works->linParms->versions, (const char *)rbuf+44, 13 );
	works->linParms->isTune=rbuf[57];
}
void storeParameters( workp works, int DataLength, unsigned char *rbuf) {
	static int initSquelch=-1;
	static int initNr=(-1);
	static int initNb=-1;
	static int initAtt=-1;
	static int initLp=-1;
	static int initAn=-1;
	static int isFirst=1;
	int i, j;
	char tuningFrequencyStr[9];
	char pitchStr[3];
	char ritStr[6];
	struct timeval ora;
	double uno,due;

	if( isFirst ) {
		memset( works->serParms, 0, sizeof( serialParms ) ); 
		isFirst = 0;
	}
	works->serParms->duoType =  ( rbuf[0] & 0x40 ) >> 6;
	works->serParms->vfoUsed = ( rbuf[0] & 0x20 ) >> 5;
	works->serParms->vfoSent = ( rbuf[0] & 0x10 ) >> 4;

        gettimeofday( &ora, NULL );
	uno=1000.0*ora.tv_sec+0.001*ora.tv_usec;

	due=1000.0*works->freqChanged.tv_sec+0.001*works->freqChanged.tv_usec;
	if( uno > due+5000 || due==0 ) {
		for( j=1; j<9; j++ ) {
			tuningFrequencyStr[j] = (char) (rbuf[j] | 0x30);
		} 
		works->serParms->tuningFrequencyRx = 0;
		for ( j = 1; j < 9; j++) {
			works->serParms->tuningFrequencyRx <<= 4;
			works->serParms->tuningFrequencyRx |= tuningFrequencyStr[j] - 0x30;
		}
		if( works->serParms->vfoSent == 0 ) {
			works->serParms->tuningFrequencyA = works->serParms->tuningFrequencyRx;
		} else {
			works->serParms->tuningFrequencyB = works->serParms->tuningFrequencyRx;
		}
		if( works->serParms->vfoUsed == 0 ) {
			works->serParms->tuningFrequency = works->serParms->tuningFrequencyA;
		} else {
			works->serParms->tuningFrequency = works->serParms->tuningFrequencyB;
		}
		works->serParms->oldfreq = works->serParms->freq;
		works->serParms->freq = works->serParms->tuningFrequency;
	}

	due=1000.0*works->modeChanged.tv_sec+0.001*works->modeChanged.tv_usec;
	if( uno > due+2500 || due == 0 ) {
		if (DataLength > 9 && (rbuf[9] & 0x80) == 0x80) {
			works->serParms->tune = (( rbuf[9] & 0x40 ) >> 6)?1:-1;
			works->serParms->split = ( rbuf[9] & 0x20 ) >> 5;
			works->serParms->splitSA = ( rbuf[9] & 0x10 ) >> 4;
			works->serParms->modeRx = rbuf[9] & 0x0F;
			if( works->serParms->vfoSent == 0 ) {
				works->serParms->modeA = works->serParms->modeRx;
			} else {
				works->serParms->modeB = works->serParms->modeRx;
			}
			if( works->serParms->vfoUsed == 0 ) {
				works->serParms->mode = works->serParms->modeA;
			} else {
				works->serParms->mode = works->serParms->modeB;
			}
		}
	}

	if (DataLength > 10 && (rbuf[10] & 0xC0) == 0x40) {
		works->serParms->indVol = (rbuf[10] & 0x30 ) >> 4;
		int j = ((rbuf[10] & 0x08)>>3);
		works->serParms->mainVol = j>0?-1:1;
		j = (rbuf[10] & 0x04)>>2;
		due=1000.0*works->auxVolChanged.tv_sec+0.001*works->auxVolChanged.tv_usec;
		if( uno > due+1500 || due == 0 ) {
			works->serParms->auxVol  = j>0?-1:1;
		}
		due=1000.0*works->ritChanged.tv_sec+0.001*works->ritChanged.tv_usec;
		if( uno > due+1500 || due == 0 ) {
			j = (rbuf[10] & 0x02)>>1;
			works->serParms->sideVol= j>0?-1:1;
		}
		j = rbuf[10] & 0x01;
		works->serParms->ptt = j>0?j:-1;
	}
	due=1000.0*works->squelchChanged.tv_sec+0.001*works->squelchChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if (DataLength > 11 && (rbuf[11] & 0xC0) == 0x40) {
			j = rbuf[11] & 0x0F;
			works->serParms->squelch = j > 0 ? j : works->serParms->squelch < 0 ? works->serParms->squelch : -works->serParms->squelch;
			if (works->serParms->squelch == 0) {
				works->serParms->squelch = initSquelch;
			}
		}
	}
	if( DataLength>13 && (rbuf[12]&0x80)==0x80 && (rbuf[13]&0xc0)==0x40 ) {
		works->serParms->agcAuto = (rbuf[12] & 0x04)>>2;
		int k = rbuf[12] & 0x03;
		due=1000.0*works->agcAlvlChanged.tv_sec+0.001*works->agcAlvlChanged.tv_usec;
		if( uno > due+1500 || due == 0 ) {
			works->serParms->agcAlvl = k > 0 ? k: works->serParms->agcAlvl < 0 ? works->serParms->agcAlvl : -works->serParms->agcAlvl;
		}
		k = ( rbuf[12] >> 3 ) & 0x0F;
		due=1000.0*works->thresholdChanged.tv_sec+0.001*works->thresholdChanged.tv_usec;
		if( uno > due+1500 || due == 0 ) {
			works->serParms->agcThr = k > 0 ? k: works->serParms->agcThr < 0 ? works->serParms->agcThr : - works->serParms->agcThr;
		}
		works->serParms->cwMute = (rbuf[13] & 0x20) >> 5;
		works->serParms->phMute = (rbuf[13] & 0x10) >> 4;
		int l = rbuf[13] & 0x0F;
		works->serParms->agcMlvl = l;
		works->serParms->agc = works->serParms->agcAuto ==0 ? k+1 : l+5;
	}
	due=1000.0*works->nrChanged.tv_sec+0.001*works->nrChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if (DataLength > 14 && (rbuf[14] & 0xF0) == 0x30) {
			j = rbuf[14] & 0x0F;
			works->serParms->nr = j > 0 ? j : works->serParms->nr < 0 ? works->serParms->nr : -works->serParms->nr;
			if (works->serParms->nr == 0) {
				works->serParms->nr = initNr;
			}
		}
	}
	due=1000.0*works->nbChanged.tv_sec+0.001*works->nbChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if (DataLength > 15 && (rbuf[15] & 0xF0) == 0x30) {
			j = rbuf[15] & 0x0F;
			works->serParms->nb = j > 0 ? j : works->serParms->nb < 0 ? works->serParms->nb : -works->serParms->nb;
			if (works->serParms->nb == 0) {
				works->serParms->nb = initNb;
			}
		}
	}
	if (DataLength > 20 && (rbuf[20] & 0xc0) == 0x40) {
		j = (rbuf[20] & 0x10) >> 4;
		works->serParms->attR = j;
		works->serParms->att = j > 0 ? j : works->serParms->att < 0 ? works->serParms->att : -works->serParms->att;
		if (works->serParms->att == 0) {
			works->serParms->att = initAtt;
		}
		j = (rbuf[20] & 0x08) >> 3;
		works->serParms->lp = j > 0 ? j : works->serParms->lp < 0 ? works->serParms->lp : -works->serParms->lp;
		if (works->serParms->lp == 0) {
			works->serParms->lp = initLp;
		}
		due=1000.0*works->anChanged.tv_sec+0.001*works->anChanged.tv_usec;
		if( uno > due+1500 || due == 0 ) {
			j = (rbuf[20] & 0x06) >> 1;
			works->serParms->an = j > 0 ? j : works->serParms->an < 0 ? works->serParms->an : -works->serParms->an;
			if (works->serParms->an == 0) {
				works->serParms->an = initAn;
			}
		}
		j = rbuf[20] & 0x01;
		works->serParms->rit = j;
		if( works->serParms->rit==0 ) {
			works->serParms->rit=(-1);
		}
	}
	if (DataLength > 21) {
		j = rbuf[21] & 0xFF;
		if( works->serParms->duoType == 1 ) {
			works->serParms->rssi = j - 192 - 31 + works->serParms->attR * 12;
		} else {
			works->serParms->rssi = j - 192 - 31 + works->serParms->attR * 10;
		}
	}
	due=1000.0*works->volumeChanged.tv_sec+0.001*works->volumeChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if( DataLength>22 ) {
			int j = rbuf[22] & 0x7F;
			if( works->serParms->indVol == 0 ) {
				works->serParms->volume = j;
			} else if( works->serParms->indVol == 1 ) {
				works->serParms->volumeAux = j > 0 ? j: works->serParms->volumeAux < 0 ? works->serParms->volumeAux: -works->serParms->volumeAux;
			} else {
				works->serParms->volumeSide = j;
			}
		}
	}
	due=1000.0*works->pitchChanged.tv_sec+0.001*works->pitchChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if (DataLength > 25) {
			pitchStr[0] = (char) (rbuf[23] | 0x30);
			pitchStr[1] = (char) (rbuf[24] | 0x30);
			pitchStr[2] = (char) (rbuf[25] | 0x30);
			int j = 0;
			j |= pitchStr[0] - 0x30;
			for ( i = 1; i < 3; i++) {
				j <<= 4;
				j |= pitchStr[i] - 0x30;
			}
			works->serParms->pitch = j;
		}
	}
	due=1000.0*works->ritChanged.tv_sec+0.001*works->ritChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if (DataLength > 30) {
			ritStr[0] = (char) (rbuf[26] | 0x30);
			ritStr[1] = (char) (rbuf[27] | 0x30);
			ritStr[2] = (char) (rbuf[28] | 0x30);
			ritStr[3] = (char) (rbuf[29] | 0x30);
			ritStr[4] = (char) (rbuf[30] | 0x30);
			ritStr[5] = (char) 0;
			j = 0;
			j |= ritStr[0] - 0x30;
			if (j > 0x07) {
				j = j - 16;
			}
			for ( i = 1; i < 5; i++) {
				j <<= 4;
				j |= ritStr[i] - 0x30;
			}
			works->serParms->ritFreq = j;
		}
	}
	due=1000.0*works->bwChanged.tv_sec+0.001*works->bwChanged.tv_usec;
	if( uno > due+1500 || due == 0 ) {
		if (DataLength > 19 && (rbuf[16] & 0xE0) == 0x60 && (rbuf[17] & 0xE0) == 0x60 && (rbuf[18] & 0xF0) == 0x30 && (rbuf[19] & 0xE0) == 0x60) {
			works->serParms->LSBbw = (rbuf[16] & 0x1F) + 1;
			works->serParms->USBbw = (rbuf[17] & 0x1F) + 1;
			works->serParms->CWbw = (rbuf[18] & 0x0F) + 1;
			works->serParms->AMbw = ((rbuf[19] & 0x1C) >> 2) + 1;
			works->serParms->FMbw = (rbuf[19] & 0x03) + 1;
			switch( works->serParms->mode ) {
				case 1:
					works->serParms->bw=1000+works->serParms->AMbw-1;
					break;
				case 2:
					works->serParms->bw=2000+works->serParms->LSBbw-1;
					break;
				case 3:
					works->serParms->bw=3000+works->serParms->USBbw-1;
					break;
				case 4:
					works->serParms->bw=4000+17-works->serParms->CWbw-1;
					break;
				case 5:
					works->serParms->bw=5000+works->serParms->FMbw-1;
					break;
				case 6:
					works->serParms->bw=6000+17-works->serParms->CWbw-1;
					break;
			}
		}
	}
	memset( rbuf, 0, DataLength );
	works->serReceived++;
}

